#include "Paracone.h"

// ==============================================================
// Airfoil coefficient functions
// Return lift, moment and zero-lift drag coefficients as a
// function of angle of attack (alpha or beta)
// ==============================================================

// 1. vertical lift component (wings and body)

void VLiftCoeff (double aoa, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*15.0;
	static const double istep = 1.0/step;
	static const int nabsc = 25;
	static const double CL[nabsc] = {0.1, 0.17, 0.2, 0.2, 0.17, 0.1, 0, -0.11, -0.24, -0.38,  -0.5,  -0.5, -0.02, 0.6355,    0.63,   0.46, 0.28, 0.13, 0.0, -0.16, -0.26, -0.29, -0.24, -0.1, 0.1};
	static const double CM[nabsc] = {  0,    0,   0,   0,    0,   0, 0,     0,    0,0.002,0.004, 0.0025,0.0012,      0,-0.0012,-0.0007,    0,    0,   0,     0,     0,     0,     0,    0,   0};
	// lift and moment coefficients from -180 to 180 in 15 degree steps.
	// This uses a documented lift slope of 0.0437/deg, everything else is rather ad-hoc

	aoa += PI;
	int idx = max (0, min (23, (int)(aoa*istep)));
	double d = aoa*istep - idx;
	*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
	*cm = CM[idx] + (CM[idx+1]-CM[idx])*d;
	*cd = 0.06 + oapiGetInducedDrag (*cl, 2.266, 0.6);
}

// 2. horizontal lift component (vertical stabiliser and body)

void HLiftCoeff (double beta, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*22.5;
	static const double istep = 1.0/step;
	static const int nabsc = 17;
	static const double CL[nabsc] = {0, 0.2, 0.3, 0.2, 0, -0.2, -0.3, -0.2, 0, 0.2, 0.3, 0.2, 0, -0.2, -0.3, -0.2, 0};

	beta += PI;
	int idx = max (0, min (15, (int)(beta*istep)));
	double d = beta*istep - idx;
	*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
	*cm = 0.0;
	*cd = 0.02 + oapiGetInducedDrag (*cl, 1.5, 0.6);
}


PARACONE::PARACONE (OBJHANDLE hObj, int fmodel)
: COMMON_VESSEL_EX (hObj)
{
	hCone = oapiLoadMeshGlobal( "Paracone");
	hDefSuit = oapiLoadMeshGlobal("mmu");
	tex   = oapiRegisterExhaustTexture ("exhaust_atrcs");

	banimsset = false;
	defaultchecked = false;
	RetroMass = 12.5;

	defaultsuit = false;
	PayRemoved = true;

//	SetDefSuit();
	
	ph_main = CreatePropellantResource( 1.0 );
	ph_retro = CreatePropellantResource (RetroMass);

	//		     				child       pos        dir       rot      handle
	Seatpos = CreateAttachment (false, OFS_SEAT,   _V(0,-1,0), _V(0,0,1), "SEAT");

}

void PARACONE::LoadState (FILEHANDLE scn, void *vs)
{
	char *line;
//	int i=0;
      
	ClearMeshesEx();
	SetStage();
	while (oapiReadScenario_nextline (scn, line)) 
	{
  		if (!strnicmp (line, "PAYLOAD", 7))
		{
			if (!defaultsuit)
				ParsePayloadString( line );
		}

		else if ( !strnicmp(line, "STAGE",5))
		{
			if (!defaultsuit)
				ParsePayloadString( line );
		}


		else 
		{
			ParseScenarioLineEx (line, vs);
		}

	}


	SetAnims();

}

void PARACONE::SaveState (FILEHANDLE scn)
{
	SaveDefaultState (scn);

	/* save payload lines */
	if (!defaultsuit)
		SaveDefaultStateEx(scn);
}

void PARACONE::SetDefSuit()
{
		defaultchecked = true;

		char checkname[512];

		strcpy (checkname, GetName()); 
		strcat (checkname, "-Suit");
		hVessel=oapiGetVesselByName(checkname);
		if (hVessel != 0) return;					// There's already a suit of this name built;
												// probably a saved scenario with the suit already
												// jettisoned.  Don't make another.

		char paystr[80];
		char classname[30];
		int i;
		strncpy( classname, GetClassName(), 29 );
		classname[29]=0;
		for( i = 0 ; i < strlen( classname ) ; i++ ) {
			classname[i] = toupper(classname[i]);
		}
		if( strstr( classname, "PARACONEGEM" ) ) {
			strncpy(paystr, "gem-eva1 gemini_leva  0.0  0.0 0.0 150.0  0.0  1.0", 79);
			PayRemoved = false;
		}
		else if( strstr( classname, "PARACONE2001" ) ) {
			strncpy(paystr, "2001/2001Astro 2001/Astro2001 0 -0.2 0.0 130.0 0.0 1.0", 79);
			RetroMass = 18.0;							// need enough fuel to de-orbit from Station V!
			DelPropellantResource (ph_retro);
			ph_retro = CreatePropellantResource (RetroMass);
			PayRemoved = false;
		}
		else if( strstr( classname, "PARACONEDG3" ) ) 
			strncpy(paystr, "DGIIIMmu DGIIIMmu 0 0.0 -0.1 200.0 0.0 1.0", 79);
		else if( strstr( classname, "PARACONE" ) )
			strncpy(paystr, "mmu nasa_mmu 0 -0.4 0.20 212.0 0.0 1.0", 79);
		else return;

		char prefix[512];

		strcpy (prefix, "PAYLOAD ");
		strcat (prefix, GetName()); 
		strcat (prefix, "-Suit ");
		strcat (prefix, paystr);

		ParsePayloadString(prefix);
		defaultsuit = true;
		CheckCentralMessage();
		SetStage();
		SetAnims();
}

void PARACONE::SetStage ()
{

	VECTOR3 pos = _V( 0,0,0 );

	ClearMeshesEx();
	COMMON_MESH tmp;
	tmp.MeshHandle(hCone);
	tmp.MeshPosition( pos);
	AddMeshEx( tmp);
	
}
void PARACONE::SetAnims ()
{

	Draw();
	if (!banimsset) {
		banimsset = true;

		anim_expand = CreateAnimation (1.0);

		static UINT ExpandGrp[2] = {2,3};
		static UINT RetroGrp[2] = {0,1};

		static MGROUP_ROTATE Retro (0, RetroGrp, 2,
			_V(0.0,0.0,-0.45), _V(1,0,0), (float)(68*RAD));
		AddAnimationComponent (anim_expand, 0.05, 0.9, &Retro);

	//	static MGROUP_TRANSLATE trans (0, ExpandGrp, 2,
	//		_V( 0.0, 0.0, 0.5));
	//	AddAnimationComponent (anim_expand, 0.68, 0.7, &trans);

		static MGROUP_SCALE Expand (0, ExpandGrp, 2,
			_V( 0.0, 0.0, 0.0), _V(1000.0,1000.0,10.0));
		AddAnimationComponent (anim_expand, 0.7, 0.75, &Expand);
	}

	SetAnimation (anim_expand, expand_proc);

	SetMeshVisibilityMode(0,MESHVIS_ALWAYS);
	VECTOR3 mesh_dir=_V(0,0.50,-0.4);
	SetCameraOffset(mesh_dir);

	
	double MainThrust = 0.0;
	double HoverThrust = 0.0;
	double RetroThrust = 2000;
	double AttThrust = 5;
	drymass = 100.0;
	EXPAND_SPEED = 0.25;
	expand_proc = 0.0;
	trans_proc = 1.0;
	numevas = 0;
	Expanding = false;

	SetEmptyMass( drymass+CalculateMass());
//	if (defaultsuit) SetEmptyMass (350.0);

	PARTICLESTREAMSPEC contrail = 
	{
		0, 5.0, 5, 250, 0.3, 0.5, 16, 2.0, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-4, 1
	};

	PARTICLESTREAMSPEC exhaust_retro = {
		0, 2.8, 40, 250, 0.1, 0.2, 42, 1.0, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-5, 0.1
	};
	exhaust_retro.tex = oapiRegisterParticleTexture ("Contrail2");
	SetRotDrag (_V(0.7,0.7,0.3)); // angular drag
//	SetEnableFocus(FALSE);

/*
	CreateAirfoil (LIFT_VERTICAL,   _V(0,0,-0.5), VLiftCoeff, 20, 270, 2.266);
	CreateAirfoil (LIFT_HORIZONTAL, _V(0,0,-4), HLiftCoeff, 20,  50, 1.5);

	CreateControlSurface (AIRCTRL_ELEVATOR, 5.0, 1.5, _V( 0, 0,  -15), AIRCTRL_AXIS_XPOS);
	CreateControlSurface (AIRCTRL_RUDDER,   2.0, 1.5, _V( 0, 3,  -16), AIRCTRL_AXIS_YPOS);
	CreateControlSurface (AIRCTRL_AILERON,  3.0, 1.5, _V( 7,-0.5,-15), AIRCTRL_AXIS_XPOS);
	CreateControlSurface (AIRCTRL_AILERON,  3.0, 1.5, _V(-7,-0.5,-15), AIRCTRL_AXIS_XNEG);
*/
	PARTICLESTREAMSPEC rps = {
		0, 20, 20, 0, 0.03, 0.5, 100, 3, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_FLAT, 1, 1, PARTICLESTREAMSPEC::ATM_PLIN, 6e7, 12e7
	};
	AddReentryStream (&rps);


	SetSize (8);
	SetTouchdownPoints (_V(0,17,-1), _V(10,10,-1), _V(-10,-10,-1));
	SetPMI (_V(2.28, 2.15, 4.09));
	SetCrossSections (_V(5.3, 5.3, 42.15));
	SetCW (0.7, 0.7, 0.9, 0.9);
	SetRotDrag (_V(0.5, 0.5, 0.3));
	SetPitchMomentScale (-1e-4);
	SetBankMomentScale (-1e-4);
	SetDefaultPropellantResource( ph_retro );
	Burning = false;
	RetroDefined = false;

	retex = oapiRegisterReentryTexture("reentry");
	SetReentryTexture(retex,6e5,1,1);



// 5 Dock #3



/*
	th_main[0] = CreateThruster (_V(0,0.0,-15.5),    _V( 0.0,0.0,1.0), 
		MainThrust, ph_main, 4e4, 4e5);

	th_main[1] = CreateThruster (_V(-3.7,0.0,-11.5), _V( 0.2,0.0,0.87), 
		MainThrust/2, ph_main, 4e4, 4e5);

	th_main[2] = CreateThruster (_V( 3.7,0.0,-11.5), _V( -0.2,0.0,0.87), 
		MainThrust/2, ph_main, 4e4, 4e5);

	thg_main = CreateThrusterGroup( th_main, 3, THGROUP_MAIN );
	AddExhaustStream (th_main[0], _V( 0.0,  0, -25.0), &exhaust_main);
	AddExhaustStream (th_main[1], _V(-2.6, 0.0,-20.0), &exhaust_main);
	AddExhaustStream (th_main[2], _V( 2.6, 0.0,-20.0), &exhaust_main);

	AddExhaust (th_main[0], 10.0, 1.0, tex);
	AddExhaust (th_main[1], 10.0, 0.5, tex);
	AddExhaust (th_main[2], 10.0, 0.5, tex);

	th_hover[0] = CreateThruster (_V(0,0,0), _V(0,1,0), HoverThrust, ph_main, 4e4, 4e5);
	thg_hover = CreateThrusterGroup (th_hover, 1, THGROUP_HOVER);

	AddExhaust (th_hover[0], 4.0, 0.2 ,_V( -0.5,   -1.0, 1.2),_V(  0.0, -1.0, 0.0), tex);
	AddExhaust (th_hover[0], 4.0, 0.2 ,_V(  0.5,   -1.0, 1.2),_V(  0.0, -1.0, 0.0), tex);
	AddExhaust (th_hover[0], 4.0, 0.2 ,_V( -0.5,   -1.0, 4.55),_V(  0.0, -1.0, 0.0), tex);
	AddExhaust (th_hover[0], 4.0, 0.2 ,_V(  0.5,   -1.0, 4.55),_V(  0.0, -1.0, 0.0), tex);
	AddExhaust (th_hover[0], 4.0, 0.2 ,_V( -0.5,   -1.0, 7.25),_V(  0.0, -1.0, 0.0), tex);
	AddExhaust (th_hover[0], 4.0, 0.2 ,_V(  0.5,   -1.0, 7.25),_V(  0.0, -1.0, 0.0), tex);
	AddExhaust (th_hover[0], 4.0, 0.2 ,_V( -0.5,   -1.2, -6.8),_V(  0.0,-1.0, 0.0), tex);
	AddExhaust (th_hover[0], 4.0, 0.2 ,_V(  0.5,   -1.2, -6.8),_V(  0.0,-1.0, 0.0), tex);
*/
	// set of attitude thrusters (idealised). The arrangement is such that no angular
	// momentum is created in linear mode, and no linear momentum is created in rotational mode.

	THRUSTER_HANDLE th_att_rot[4], th_att_lin[4];
	th_att_rot[0] = CreateThruster (_V(0,0, 15.5), _V(0, 1,0), AttThrust, ph_main, 4e4, 4e5);
	th_att_rot[1] = CreateThruster (_V(0,0,-15.5), _V(0,-1,0), AttThrust, ph_main, 4e4, 4e5);
	th_att_rot[2] = CreateThruster (_V(0,0, 15.5), _V(0,-1,0), AttThrust, ph_main, 4e4, 4e5);
	th_att_rot[3] = CreateThruster (_V(0,0,-15.5), _V(0, 1,0), AttThrust, ph_main, 4e4, 4e5);
	CreateThrusterGroup (th_att_rot,   2, THGROUP_ATT_PITCHUP);
	CreateThrusterGroup (th_att_rot+2, 2, THGROUP_ATT_PITCHDOWN);

	th_att_lin[0] = CreateThruster (_V(0,0, 15.5), _V(0, 1,0), AttThrust, ph_main, 4e4, 4e5);
	th_att_lin[3] = CreateThruster (_V(0,0,-15.5), _V(0,-1,0), AttThrust, ph_main, 4e4, 4e5);
	th_att_lin[2] = CreateThruster (_V(0,0, 15.5), _V(0,-1,0), AttThrust, ph_main, 4e4, 4e5);
	th_att_lin[1] = CreateThruster (_V(0,0,-15.5), _V(0, 1,0), AttThrust, ph_main, 4e4, 4e5);
	CreateThrusterGroup (th_att_lin,   2, THGROUP_ATT_UP);
	CreateThrusterGroup (th_att_lin+2, 2, THGROUP_ATT_DOWN);

	th_att_rot[0] = CreateThruster (_V(0,0, 15.5), _V(-1,0,0), AttThrust, ph_main, 4e4, 4e5);
	th_att_rot[1] = CreateThruster (_V(0,0,-15.5), _V( 1,0,0), AttThrust, ph_main, 4e4, 4e5);
	th_att_rot[2] = CreateThruster (_V(0,0, 15.5), _V( 1,0,0), AttThrust, ph_main, 4e4, 4e5);
	th_att_rot[3] = CreateThruster (_V(0,0,-15.5), _V(-1,0,0), AttThrust, ph_main, 4e4, 4e5);
	CreateThrusterGroup (th_att_rot,   2, THGROUP_ATT_YAWLEFT);
	CreateThrusterGroup (th_att_rot+2, 2, THGROUP_ATT_YAWRIGHT);

	th_att_lin[0] = CreateThruster (_V(0,0, 15.5), _V(-1,0,0), AttThrust, ph_main, 4e4, 4e5);
	th_att_lin[3] = CreateThruster (_V(0,0,-15.5), _V( 1,0,0), AttThrust, ph_main, 4e4, 4e5);
	th_att_lin[2] = CreateThruster (_V(0,0, 15.5), _V( 1,0,0), AttThrust, ph_main, 4e4, 4e5);
	th_att_lin[1] = CreateThruster (_V(0,0,-15.5), _V(-1,0,0), AttThrust, ph_main, 4e4, 4e5);
	CreateThrusterGroup (th_att_lin,   2, THGROUP_ATT_LEFT);
	CreateThrusterGroup (th_att_lin+2, 2, THGROUP_ATT_RIGHT);

	th_att_rot[0] = CreateThruster (_V( 2.7,0,0), _V(0, 1,0), AttThrust, ph_main, 4e4, 4e5);
	th_att_rot[1] = CreateThruster (_V(-2.7,0,0), _V(0,-1,0), AttThrust, ph_main, 4e4, 4e5);
	th_att_rot[2] = CreateThruster (_V(-2.7,0,0), _V(0, 1,0), AttThrust, ph_main, 4e4, 4e5);
	th_att_rot[3] = CreateThruster (_V( 2.7,0,0), _V(0,-1,0), AttThrust, ph_main, 4e4, 4e5);
	CreateThrusterGroup (th_att_rot,   2, THGROUP_ATT_BANKLEFT);
	CreateThrusterGroup (th_att_rot+2, 2, THGROUP_ATT_BANKRIGHT);

	th_att_lin[0] = CreateThruster (_V(0,0,-16), _V(0,0, 1), AttThrust, ph_main, 4e4, 4e5);
	th_att_lin[1] = CreateThruster (_V(0,0, 16), _V(0,0,-1), AttThrust, ph_main, 4e4, 4e5);
	CreateThrusterGroup (th_att_lin,   1, THGROUP_ATT_FORWARD);
	CreateThrusterGroup (th_att_lin+1, 1, THGROUP_ATT_BACK);


}

void PARACONE::SetClassCaps (FILEHANDLE cfg)
{

	CheckCentralMessage();

	SetStage();
	SetAnims();
}


void PARACONE::TimeStep( double simt )
{

	if( CheckCentralMessage() ) {
		while( CheckCentralMessage() ) ; {
			SetStage();
			Draw();
		}
	}

	OBJHANDLE hV = GetAttachmentStatus(Seatpos);
	if (hV) {
		VESSEL *v = oapiGetVesselInterface (hV);
		double strappedmass = v->GetMass();
		SetEmptyMass (drymass+strappedmass+CalculateMass());
		defaultchecked = true;
	}

	if (!defaultchecked)
		if (GetMass() == (drymass + GetPropellantMass(ph_main) + GetPropellantMass(ph_retro)))		// No suit; just pack and propellant
			SetDefSuit();

	if (!PayRemoved) {
		PayRemoved = true;
		defaultchecked = true;
		OverridePayloadVelocity(_V(0,0,0));
		depV = DeployPayload();
		SetStage();
		Draw();
		SetEmptyMass( drymass+CalculateMass());
		if (depV)
			StrapUnstrap();
	}

	if ((expand_proc >= 1.0) && (!RetroDefined)) {  // Define the retro motor for once and only

		double RetroThrust = 2000;

		PARTICLESTREAMSPEC contrail = 
		{
			0, 0.25, 1000, 150, 0.3, 0.25, 16, 0.0, PARTICLESTREAMSPEC::EMISSIVE,
			PARTICLESTREAMSPEC::LVL_PLIN, 0, 1,
			PARTICLESTREAMSPEC::ATM_PLIN, -0.01, 0.1
		};


		retro_engine[0] = CreateThruster (_V(0,0.0,1.0),    _V( 0.0,0.0,-1.0), 
		RetroThrust, ph_retro, 2500, 2500);
		AddExhaust(retro_engine[0],1.0,0.1, _V(0,0.02,0.85), _V(0,0,1),tex);
		thg_retro = CreateThrusterGroup(retro_engine,1,THGROUP_RETRO);
		AddExhaustStream (retro_engine[0], _V(0,0,1.0), &contrail);
		RetroDefined = true;
	}
	if (RetroDefined)
		if (GetThrusterGroupLevel (thg_retro) > 0.0) 
			Burning = true;
	if (Burning) SetThrusterGroupLevel (thg_retro,1.0);  // Act as a solid motor (burn to depletion)

	SetAnimation (anim_expand, expand_proc);
	double da = oapiGetSimStep() * EXPAND_SPEED;
	if (Expanding) {
		expand_proc = expand_proc + da;
	}
	if (expand_proc >= 1.0) {
		Expanding = false;
		expand_proc = 1.0;
	}
}


void PARACONE::StrapUnstrap()
{

			VECTOR3 gpos, grms, pos, dir, rot;
			Local2Global (OFS_SEAT, grms);  // global position of RMS tip
			
			// Search the complete vessel list for a grappling candidate.
			// Not very scalable ...

			for (DWORD i = 0; i < oapiGetVesselCount(); i++) {
				OBJHANDLE hV = oapiGetVesselByIndex (i);
				if (hV == GetHandle()) continue; // we don't want to grapple ourselves ...
				oapiGetGlobalPos (hV, &gpos);
//				sprintf(oapiDebugString(),"Distance is, %d, Too far to strap in!",dist(gpos, grms));
				if (dist (gpos, grms) < oapiGetSize (hV)) { // in range
					
//					sprintf(oapiDebugString(),"Distance is too far to strap in!");

					VESSEL *v = oapiGetVesselInterface (hV);
					DWORD nAttach = v->AttachmentCount (true);
					for (DWORD j = 0; j < nAttach; j++) { // now scan all attachment points of the candidate
						ATTACHMENTHANDLE hAtt = v->GetAttachmentHandle (true, j);
						const char *id = v->GetAttachmentId (hAtt);
						if (strncmp (id, "BUTT", 4)) continue; // attachment point not compatible
						v->GetAttachmentParams (hAtt, pos, dir, rot);
						v->Local2Global (pos, gpos);
						if (dist (gpos, grms) < MAX_GRAPPLING_DIST) { // found one!
							// check whether satellite is currently clamped into payload bay
							if (hV == GetAttachmentStatus (Seatpos))
							{
								DetachChild (Seatpos);
//								sprintf(oapiDebugString(),"Unstrapped");
								oapiSetFocusObject (hV);
								SetEmptyMass (drymass);
							}
							else
							{
								AttachChild (hV, Seatpos, hAtt);
//								sprintf(oapiDebugString(),"Strapped in!");	
							}
							return;
						}
					}
				}
			
			}
}


int PARACONE::ConsumeKey ( const char *keystate)
{
	return 0;
}

int PARACONE::ConsumeBufferedKey ( DWORD dwKey, bool fPressed, const char *keystate)
{

		if (KEYDOWN (keystate, OAPI_KEY_K)) 
		{ // "Unfold"
			if (oapiAcceptDelayedKey (OAPI_KEY_K, 1.0))
			{
			//	OperateDoors (door_status == DOOR_CLOSED || door_status == DOOR_CLOSING ?
			//		DOOR_OPENING : DOOR_CLOSING);
				Expanding = true;
			}
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_J)) { // "Jettison Stage"
			if (oapiAcceptDelayedKey (OAPI_KEY_J, 1.0)) {
				OverridePayloadVelocity(_V(0,0,0));
				depV = DeployPayload();
				SetStage();
				Draw();
				SetEmptyMass( drymass+CalculateMass());
				if (depV) {
					hVessel=oapiGetVesselByName(depV->GetName());
					oapiSetFocusObject(hVessel);
				}
			}
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_S)) { // "Strap/Unstrap" 
			if (oapiAcceptDelayedKey (OAPI_KEY_S, 1.0)) {

				StrapUnstrap();
			
			}
		}

/*
		if (KEYDOWN (keystate, OAPI_KEY_E)) 
		{ // "Jettison Stage"
			if (oapiAcceptDelayedKey (OAPI_KEY_E, 1.0))
			{
			

				VESSELSTATUS vs;
				OBJHANDLE hvessel;
				GetStatus (vs);
				vs.eng_main = vs.eng_hovr = 0.0;
				vs.fuel=1.0;
				vs.status=0;
				VECTOR3 ofs = _V(3.5,0.0,17.25);
				Local2Rel (ofs, vs.rpos);
				char name[256];
				numevas++;
				sprintf(name, "EVA-%d", numevas);
	
				VECTOR3 vel = _V(0,0.2,1.0);
				VECTOR3 rofs, rvel = {vs.rvel.x, vs.rvel.y, vs.rvel.z};
				Local2Rel (ofs, vs.rpos);
				GlobalRot (vel, rofs);
				vs.rvel.x = rvel.x+rofs.x;
				vs.rvel.y = rvel.y+rofs.y;
				vs.rvel.z = rvel.z+rofs.z;
				VESSEL::Create (name, "EVA", vs);
				hvessel=oapiGetVesselByName(name);
				oapiSetFocusObject(hvessel);
	
			}
			return 1;
		}
*/

		return 0;
}



/* basic vessel class setup */

void PARACONE::clbkSetClassCaps (FILEHANDLE cfg)
{
	SetClassCaps(cfg);
}

void PARACONE::clbkLoadStateEx (FILEHANDLE scn, void *vs)
{
	LoadState(  scn, vs );
}

void PARACONE::clbkSaveState (FILEHANDLE scn)
{
	SaveState(  scn );
}

void PARACONE::clbkPostStep (double simt, double simdt, double mjd)
{
	TimeStep(simt);
}

void PARACONE::clbkVisualCreated (VISHANDLE _vis, int refcount)
{
	SetStage( );
}

void PARACONE::clbkVisualDestroyed (VISHANDLE _vis, int refcount)
{
	SetStage( );
}

int PARACONE::clbkConsumeBufferedKey (DWORD dwKey, bool fPressed, char *keystate)
{
	int ret = ConsumeBufferedKey( dwKey, fPressed, keystate );
	return ret;
}


// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new PARACONE (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (PARACONE*)vessel;
}

